CON

  _CLKMODE = XTAL1 + PLL16X
  _XINFREQ = 5_000_000  
  
  #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR
  #14, GOTOX, GOTOY, CLS

  BUFFER_SIZE = 80

  'Transmission and reception constants
  STX = 2 'Start Of Text
  ETX = 3 'End Of Text
  ENQ = 5 'Enquire the result of the transmission
  ACK = 6 'Positive result of a successful transmission
  NAK = 21 'Negative result of an unsuccessful transmission
  US = 31 'Data block seperator which is used for parsing   
  
VAR

  BYTE CmdBuffer[BUFFER_SIZE]
  BYTE InstBuffer[BUFFER_SIZE]
  BYTE CrcBuffer[BUFFER_SIZE]
  BYTE Char
    
OBJ

  FDS : "Extended_FDSerial"

PUB Main | Index

  FDS.Start(31, 30, 0, 115_200)

  WAITCNT(CLKFREQ / 1_000 + CNT)
  
  FDS.Tx(CLS)
  FDS.RxFlush
  
  REPEAT UNTIL FDS.RxDec == STX 'Start of data transmission
  
  FDS.Str(STRING("STX Located - Start filling the buffer.", CR))
  
  BYTEFILL(@CmdBuffer, 0, BUFFER_SIZE)
  
  Index := 0
  
  repeat while FDS.RxDec <> ETX 'End of data transmission

    'This code works with the following two assignments commented out.  My theory
    'is that if I can find FDS.RxDec as proven in this code, then I should be able
    'to get that byte into the CmdBuffer array, and then display it.

    'Char := FDS.RxDec  
    'CmdBuffer[Index] := Char

    Index++

  FDS.str(string("ETX Located - Buffer filling has ended.", CR))

PUB ComputeCRC(Ptr, Num)
{
  This snippet was contributed by Dave Hein.  It performs a Cyclic Redundancy Check
  on sent and received data.  It's main purpose is to ascertain the intregrity of
  transmitted data and here is how it works.

  Source:   Ritter, Terry (February 1986)

  A CRC-enabled device calculates a short, fixed-length binary sequence, known as
  the CRC code or just CRC for each block of data, and sends or stores them both
  together. When a block is read or received the device repeats the calculation;
  if the new CRC does not match the one calculated earlier, then the block contains
  a data error and the device may take corrective action such as rereading or
  requesting the block be sent again, otherwise the data is assumed to be error
  free, though, with some small probability, it may contain undetected errors.  

  Dave Hein said:

  "The code that I posted will generate the stadard ITU 16-bit CRC for a block
  of "num" bytes starting at the address in "ptr". This routine assumes that
  the data is transmitted LSB first. If the data is transmitted MSB first a
  different routine would be needed, which would use a polynomial word of
  $8408 instead of $1021, which is the bit-reversed version of $8408.
  If you want to use this to generate a 16-bit hash code of a string you would
  do something like hash := ComputeCRC(@strbuf, strsize(@strbuf))"
}
  
  REPEAT Num
  
    RESULT ^= (BYTE[Ptr++] << 8)
    
    REPEAT 8
    
      RESULT <<= 1
      
      IF RESULT & $10000
      
        RESULT ^= $1021
        
  RESULT &= $FFFF
  